import { L as LexicalList_1 } from '../../node_modules/@lexical/list/LexicalList.js';
import { useLexicalComposerContext } from '@lexical/react/LexicalComposerContext';
import { INDENT_CONTENT_COMMAND, COMMAND_PRIORITY_HIGH, $getSelection, $isRangeSelection, $isElementNode } from 'lexical';
import { useEffect } from 'react';

function getElementNodesInSelection(selection) {
    const nodesInSelection = selection.getNodes();
    if (nodesInSelection.length === 0) {
        return new Set([
            selection.anchor.getNode().getParentOrThrow(),
            selection.focus.getNode().getParentOrThrow()
        ]);
    }
    return new Set(nodesInSelection.map((n) => ($isElementNode(n) ? n : n.getParentOrThrow())));
}
function isIndentPermitted(maxDepth) {
    const selection = $getSelection();
    if (!$isRangeSelection(selection)) {
        return false;
    }
    const elementNodesInSelection = getElementNodesInSelection(selection);
    let totalDepth = 0;
    for (const elementNode of elementNodesInSelection) {
        if (LexicalList_1.$isListNode(elementNode)) {
            totalDepth = Math.max(LexicalList_1.$getListDepth(elementNode) + 1, totalDepth);
        }
        else if (LexicalList_1.$isListItemNode(elementNode)) {
            const parent = elementNode.getParent();
            if (!LexicalList_1.$isListNode(parent)) {
                throw new Error("ListMaxIndentLevelPlugin: A ListItemNode must have a ListNode for a parent.");
            }
            totalDepth = Math.max(LexicalList_1.$getListDepth(parent) + 1, totalDepth);
        }
    }
    return totalDepth <= maxDepth;
}
function ListMaxIndentLevelPlugin({ maxDepth }) {
    const [editor] = useLexicalComposerContext();
    useEffect(() => {
        return editor.registerCommand(INDENT_CONTENT_COMMAND, () => !isIndentPermitted(maxDepth !== null && maxDepth !== void 0 ? maxDepth : 7), COMMAND_PRIORITY_HIGH);
    }, [editor, maxDepth]);
    return null;
}

export { ListMaxIndentLevelPlugin as default };
